import { jsxs, jsx, Fragment } from 'react/jsx-runtime';
import { cx } from '@emotion/css';
import { useVirtualizer } from '@tanstack/react-virtual';
import { useCombobox } from 'downshift';
import { debounce } from 'lodash';
import { useState, useMemo, useId, useCallback } from 'react';
import '@grafana/data';
import { useStyles2 } from '../../themes/ThemeContext.js';
import 'micro-memoize';
import '@emotion/react';
import 'tinycolor2';
import '../../utils/skeleton.js';
import { t, Trans } from '../../utils/i18n.js';
import { Icon } from '../Icon/Icon.js';
import { AutoSizeInput } from '../Input/AutoSizeInput.js';
import { Input } from '../Input/Input.js';
import { Box } from '../Layout/Box/Box.js';
import { Stack } from '../Layout/Stack/Stack.js';
import { ScrollContainer } from '../ScrollContainer/ScrollContainer.js';
import { getComboboxStyles } from './getComboboxStyles.js';
import { OPTION_HEIGHT, useComboboxFloat } from './useComboboxFloat.js';
import { useLatestAsyncCall, StaleResultError } from './useLatestAsyncCall.js';

function itemToString(item) {
  var _a, _b;
  return (_b = (_a = item == null ? void 0 : item.label) != null ? _a : item == null ? void 0 : item.value.toString()) != null ? _b : "";
}
function itemFilter(inputValue) {
  const lowerCasedInputValue = inputValue.toLowerCase();
  return (item) => {
    var _a, _b;
    return !inputValue || ((_a = item == null ? void 0 : item.label) == null ? void 0 : _a.toLowerCase().includes(lowerCasedInputValue)) || ((_b = item == null ? void 0 : item.value) == null ? void 0 : _b.toString().toLowerCase().includes(lowerCasedInputValue));
  };
}
const asyncNoop = () => Promise.resolve([]);
const Combobox = ({
  options,
  onChange,
  value: valueProp,
  isClearable = false,
  createCustomValue = false,
  id,
  width,
  "aria-labelledby": ariaLabelledBy,
  ...restProps
}) => {
  const value = typeof valueProp === "object" ? valueProp == null ? void 0 : valueProp.value : valueProp;
  const isAsync = typeof options === "function";
  const loadOptions = useLatestAsyncCall(isAsync ? options : asyncNoop);
  const [asyncLoading, setAsyncLoading] = useState(false);
  const [asyncError, setAsyncError] = useState(false);
  const [items, setItems] = useState(isAsync ? [] : options);
  const selectedItemIndex = useMemo(() => {
    if (isAsync) {
      return null;
    }
    if (value === null) {
      return null;
    }
    const index = options.findIndex((option) => option.value === value);
    if (index === -1) {
      return null;
    }
    return index;
  }, [options, value, isAsync]);
  const selectedItem = useMemo(() => {
    if (selectedItemIndex !== null && !isAsync) {
      return options[selectedItemIndex];
    }
    return typeof valueProp === "object" ? valueProp : { value: valueProp, label: valueProp.toString() };
  }, [selectedItemIndex, isAsync, valueProp, options]);
  const menuId = `downshift-${useId().replace(/:/g, "--")}-menu`;
  const labelId = `downshift-${useId().replace(/:/g, "--")}-label`;
  const styles = useStyles2(getComboboxStyles);
  const virtualizerOptions = {
    count: items.length,
    getScrollElement: () => scrollRef.current,
    estimateSize: () => OPTION_HEIGHT,
    overscan: 4
  };
  const rowVirtualizer = useVirtualizer(virtualizerOptions);
  const debounceAsync = useMemo(
    () => debounce((inputValue, customValueOption) => {
      loadOptions(inputValue).then((opts) => {
        setItems(customValueOption ? [customValueOption, ...opts] : opts);
        setAsyncLoading(false);
        setAsyncError(false);
      }).catch((err) => {
        if (!(err instanceof StaleResultError)) {
          setAsyncError(true);
          setAsyncLoading(false);
        }
      });
    }, 200),
    [loadOptions]
  );
  const {
    getInputProps,
    getMenuProps,
    getItemProps,
    isOpen,
    highlightedIndex,
    setInputValue,
    openMenu,
    closeMenu,
    selectItem
  } = useCombobox({
    menuId,
    labelId,
    inputId: id,
    items,
    itemToString,
    selectedItem,
    onSelectedItemChange: ({ selectedItem: selectedItem2 }) => {
      onChange(selectedItem2);
    },
    defaultHighlightedIndex: selectedItemIndex != null ? selectedItemIndex : 0,
    scrollIntoView: () => {
    },
    onInputValueChange: ({ inputValue }) => {
      const customValueOption = createCustomValue && inputValue && items.findIndex((opt) => opt.label === inputValue || opt.value === inputValue) === -1 ? {
        // Type casting needed to make this work when T is a number
        value: inputValue,
        description: t("combobox.custom-value.create", "Create custom value")
      } : null;
      if (isAsync) {
        if (customValueOption) {
          setItems([customValueOption]);
        }
        setAsyncLoading(true);
        debounceAsync(inputValue, customValueOption);
        return;
      }
      const filteredItems = options.filter(itemFilter(inputValue));
      setItems(customValueOption ? [customValueOption, ...filteredItems] : filteredItems);
    },
    onIsOpenChange: ({ isOpen: isOpen2, inputValue }) => {
      if (isOpen2 && !isAsync) {
        setItems(options);
        return;
      }
      if (isOpen2 && isAsync) {
        setAsyncLoading(true);
        loadOptions(inputValue != null ? inputValue : "").then((options2) => {
          setItems(options2);
          setAsyncLoading(false);
          setAsyncError(false);
        }).catch((err) => {
          if (!(err instanceof StaleResultError)) {
            setAsyncError(true);
            setAsyncLoading(false);
          }
        });
        return;
      }
    },
    onHighlightedIndexChange: ({ highlightedIndex: highlightedIndex2, type }) => {
      if (type !== useCombobox.stateChangeTypes.MenuMouseLeave) {
        rowVirtualizer.scrollToIndex(highlightedIndex2);
      }
    }
  });
  const { inputRef, floatingRef, floatStyles, scrollRef } = useComboboxFloat(items, rowVirtualizer.range, isOpen);
  const onBlur = useCallback(() => {
    var _a, _b;
    setInputValue((_b = (_a = selectedItem == null ? void 0 : selectedItem.label) != null ? _a : value == null ? void 0 : value.toString()) != null ? _b : "");
  }, [selectedItem, setInputValue, value]);
  const handleSuffixClick = useCallback(() => {
    isOpen ? closeMenu() : openMenu();
  }, [isOpen, openMenu, closeMenu]);
  const InputComponent = width === "auto" ? AutoSizeInput : Input;
  const suffixIcon = asyncLoading ? "spinner" : (
    // If it's loading, show loading icon. Otherwise, icon indicating menu state
    isOpen ? "search" : "angle-down"
  );
  return /* @__PURE__ */ jsxs("div", { children: [
    /* @__PURE__ */ jsx(
      InputComponent,
      {
        width: width === "auto" ? void 0 : width,
        suffix: /* @__PURE__ */ jsxs(Fragment, { children: [
          !!value && value === (selectedItem == null ? void 0 : selectedItem.value) && isClearable && /* @__PURE__ */ jsx(
            Icon,
            {
              name: "times",
              className: styles.clear,
              title: t("combobox.clear.title", "Clear value"),
              tabIndex: 0,
              role: "button",
              onClick: () => {
                selectItem(null);
              },
              onKeyDown: (e) => {
                if (e.key === "Enter" || e.key === " ") {
                  selectItem(null);
                }
              }
            }
          ),
          /* @__PURE__ */ jsx(Icon, { name: suffixIcon, onClick: handleSuffixClick })
        ] }),
        ...restProps,
        ...getInputProps({
          ref: inputRef,
          /*  Empty onCall to avoid TS error
           *  See issue here: https://github.com/downshift-js/downshift/issues/718
           *  Downshift repo: https://github.com/downshift-js/downshift/tree/master
           */
          onChange: () => {
          },
          onBlur,
          "aria-labelledby": ariaLabelledBy
          // Label should be handled with the Field component
        })
      }
    ),
    /* @__PURE__ */ jsx(
      "div",
      {
        className: cx(styles.menu, !isOpen && styles.menuClosed),
        style: {
          ...floatStyles
        },
        ...getMenuProps({
          ref: floatingRef,
          "aria-labelledby": ariaLabelledBy
        }),
        children: /* @__PURE__ */ jsxs(ScrollContainer, { showScrollIndicators: true, maxHeight: "inherit", ref: scrollRef, children: [
          isOpen && !asyncError && /* @__PURE__ */ jsx("ul", { style: { height: rowVirtualizer.getTotalSize() }, className: styles.menuUlContainer, children: rowVirtualizer.getVirtualItems().map((virtualRow) => {
            var _a;
            return /* @__PURE__ */ jsx(
              "li",
              {
                "data-index": virtualRow.index,
                className: cx(
                  styles.option,
                  selectedItem && items[virtualRow.index].value === selectedItem.value && styles.optionSelected,
                  highlightedIndex === virtualRow.index && styles.optionFocused
                ),
                style: {
                  height: virtualRow.size,
                  transform: `translateY(${virtualRow.start}px)`
                },
                ...getItemProps({
                  item: items[virtualRow.index],
                  index: virtualRow.index
                }),
                children: /* @__PURE__ */ jsxs("div", { className: styles.optionBody, children: [
                  /* @__PURE__ */ jsx("span", { className: styles.optionLabel, children: (_a = items[virtualRow.index].label) != null ? _a : items[virtualRow.index].value }),
                  items[virtualRow.index].description && /* @__PURE__ */ jsx("span", { className: styles.optionDescription, children: items[virtualRow.index].description })
                ] })
              },
              `${items[virtualRow.index].value}-${virtualRow.index}`
            );
          }) }),
          /* @__PURE__ */ jsxs("div", { "aria-live": "polite", children: [
            asyncError && /* @__PURE__ */ jsxs(MessageRow, { children: [
              /* @__PURE__ */ jsx(Icon, { name: "exclamation-triangle", size: "md", className: styles.warningIcon }),
              /* @__PURE__ */ jsx(Trans, { i18nKey: "combobox.async.error", children: "An error occurred while loading options." })
            ] }),
            items.length === 0 && !asyncError && /* @__PURE__ */ jsx(MessageRow, { children: /* @__PURE__ */ jsx(Trans, { i18nKey: "combobox.options.no-found", children: "No options found." }) })
          ] })
        ] })
      }
    )
  ] });
};
const MessageRow = ({ children }) => {
  return /* @__PURE__ */ jsx(Box, { padding: 2, color: "secondary", children: /* @__PURE__ */ jsx(Stack, { justifyContent: "center", alignItems: "center", children }) });
};

export { Combobox };
//# sourceMappingURL=Combobox.js.map
